package com.gct.tool.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.tool.entity.centralbase.CbCdWellSource;
import com.gct.tool.entity.centralbase.CbPcProWellboreVolDaily;
import com.gct.tool.entity.sql.QueryBody;
import com.gct.tool.feign.DataBankClient;
import com.gct.tool.feign.DataBankTokenClient;
import com.gct.tool.feign.WorkDiagramClient;
import com.gct.tool.mapper.centralbase.CbCdWellSourceMapper;
import com.gct.tool.mapper.centralbase.CbPcProWellboreVolDailyMapper;
import lombok.extern.slf4j.Slf4j;
import org.mockito.internal.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class LoadBaseDataService {

    final String governDataServiceIdentity = "CDF03A45C9BF4D9B85155364FAAEE08C14D85F6D51774A59GMXF6MBQG4ZDOKZXL4YF6KZSGEZGEKZWMBQTMKZVL43WCYJVG4XWCMLDGUXT22DKO43TAZ3CMYXXEL3AMFWWWN3ROFXGEZZSONYDAMBTGEYTKLRVGEZTONZXFYXTAMBSF4YDAMQ";
    final String sharedDataServiceIdentity = "77ACF8C095A94D3CA3474B9561FEA62D352F67FAE44AF04CMA2TGLZPMJPTGKZUF4XTIKZSGRRDGK27MIZGAKZOF5RDGNZTGBQS4YRWF4XT2NDFGZ2DCNLIFYYXEY3XMBVG4NRSNR4DGZTEMRWTAMBTGEZC4MBQGUXTONZXFYXTAMBSF4YDAMQ";

    @Autowired
    BaseResourceService baseResourceService;

    @Autowired
    DataBankClient dataBankClient;
    @Autowired
    DataBankTokenClient tokenClient;

    @Autowired
    WorkDiagramClient workDiagramClient;

    @Autowired
    CbCdWellSourceMapper cbCdWellSourceMapper;

    @Autowired
    CbPcProWellboreVolDailyMapper cbPcProWellboreVolDailyMapper;

    private void printMap(Object o, boolean isPrintNull, Set<String> filterSet) {
        ((Map) o).forEach((k, v) -> {
            if ((isPrintNull || Objects.nonNull(v)) && (Objects.isNull(filterSet) || filterSet.contains(k)))
                System.out.printf("%s:%s\n", k, v);
        });
    }

    /**
     * @return
     */
    public void loadCdWellSource() {
        List<String> wellIds = baseResourceService.getWellIds();
//        wellIds = wellIds.subList(0, 1);
        Set<String> filterSet = new HashSet<>();
        filterSet.addAll(Arrays.asList("WELL_ID", "WELL_COMMON_NAME", "WELL_LEGAL_NAME"));
        wellIds.forEach(wellId -> {
            QueryBody queryBody = new QueryBody();
            queryBody.select("TL").eq("WELL_LEGAL_NAME", wellId);
            JSONObject object = dataBankClient.getCdWellData(queryBody.toJsonMap(), 0, 10, governDataServiceIdentity);
            System.out.println(object);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");
//            ((Map) array.get(0)).forEach((k,v)-> System.out.printf(k+","));
            System.out.println("");
            array.forEach(o -> {
                printMap(o, false, null);
//                System.out.printf("%s,%s\n",((Map) o).get("WELL_LEGAL_NAME"),((Map) o).get("WELL_ID"));
            });
        });
    }

    public void loadVolDailyData() {
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
        cbCdWellSources.forEach(cbCdWellSource -> {
            QueryBody queryBody = new QueryBody();
            String dateStr = LocalDateTime.now().minusDays(3).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryBody.select("TL").eq("WELL_ID", cbCdWellSource.getWellId()).gt("PROD_DATE", dateStr);
            JSONObject object = dataBankClient.getVolDailyData(queryBody.toJsonMap(), 0, 5, governDataServiceIdentity);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");
            for (int i = 0; i < array.size(); i++) {
                JSONObject volDailyJson = array.getJSONObject(i);
                CbPcProWellboreVolDaily volDaily = volDailyJson.toJavaObject(CbPcProWellboreVolDaily.class);
                volDaily.setWaterCut(volDailyJson.getDouble("WATER_CUT_RATIO"));
                QueryWrapper<CbPcProWellboreVolDaily> wrapper = new QueryWrapper<CbPcProWellboreVolDaily>()
                        .eq("well_id", volDaily.getWellId())
                        .eq("prod_date", volDaily.getProdDate());
                if (Objects.isNull(cbPcProWellboreVolDailyMapper.selectOne(wrapper))) {
                    cbPcProWellboreVolDailyMapper.insert(volDaily);
                } else {
                    cbPcProWellboreVolDailyMapper.update(volDaily, wrapper);
                }
            }

        });
    }

    public void loadStatusDailyData() {
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
        cbCdWellSources = cbCdWellSources.subList(0, 1);
        cbCdWellSources.forEach(cbCdWellSource -> {
            QueryBody queryBody = new QueryBody();
            String dateStr = LocalDateTime.now().minusDays(365).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryBody.select("TL").eq("WELL_ID", cbCdWellSource.getWellId());
//            queryBody.select("TL");
            JSONObject object = dataBankClient.getStatusDailyData(queryBody.toJsonMap(), 0, 500, sharedDataServiceIdentity);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");
            HashSet<String> filterSet = new HashSet<>();
            filterSet.add("OIL_NOZZLE");
            filterSet.add("START_PUMP_DEPTH");
            filterSet.add("START_PUMP_LIQ_LEVEL");
            filterSet.add("END_PUMP_DEPTH");
            filterSet.add("END_PUMP_LIQ_LEVEL");
            array.forEach(o -> printMap(o, true, null));
//            for (int i = 0; i < array.size(); i++) {
//                JSONObject volDailyJson = array.getJSONObject(i);
//                CbPcProWellboreVolDaily volDaily = volDailyJson.toJavaObject(CbPcProWellboreVolDaily.class);
//                volDaily.setWaterCut(volDailyJson.getDouble("WATER_CUT_RATIO"));
//                QueryWrapper<CbPcProWellboreVolDaily> wrapper = new QueryWrapper<CbPcProWellboreVolDaily>()
//                        .eq("well_id", volDaily.getWellId())
//                        .eq("prod_date", volDaily.getProdDate());
//                if (Objects.isNull( cbPcProWellboreVolDailyMapper.selectOne(wrapper))){
//                    cbPcProWellboreVolDailyMapper.insert(volDaily);
//                } else {
//                    cbPcProWellboreVolDailyMapper.update(volDaily,wrapper);
//                }
//            }
        });
    }

    private JSONArray executeRemoteSql(String sqlString){
        System.out.println(sqlString);
        String res = workDiagramClient.getWorkDiagramData(sqlString);
        res=res.replace("\\","");
        res=res.substring(1,res.length()-1);
        JSONObject selectMaxTimeJson = JSONObject.parseObject(res);
        return selectMaxTimeJson.getJSONArray("dataList");
    }

    public void loadWorkDiagramData(){
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
        cbCdWellSources = cbCdWellSources.subList(0, 1);
        cbCdWellSources.forEach(cbCdWellSource -> {
            String selectNearlyWorkDiagramSqlStr = String.format(
                            "select * from PC_FD_PUMPJACK_DYNA_DIA_T@A11gxk where DYNA_CREATE_TIME =  " +
                            "(select max(DYNA_CREATE_TIME) from PC_FD_PUMPJACK_DYNA_DIA_T@A11gxk where WELL_COMMON_NAME = '%s')" +
                            "and WELL_COMMON_NAME = '%s'",
                    cbCdWellSource.getWellLegalName(),cbCdWellSource.getWellLegalName());
            JSONObject nearlyWorkDiagram = executeRemoteSql(selectNearlyWorkDiagramSqlStr).getJSONObject(0);
            String displacement = nearlyWorkDiagram.getString("DISPLACEMENT");
            String load = nearlyWorkDiagram.getString("DISP_LOAD");
            List<Double[]> sgt = new ArrayList<>();
            List<Double> dispacementList = Arrays.stream(displacement.split(";")).map(Double::parseDouble).collect(Collectors.toList());
            List<Double> loadList = Arrays.stream(load.split(";")).map(Double::parseDouble).collect(Collectors.toList());
            for (int i=0;i<Math.min(dispacementList.size(),loadList.size());i++){
                sgt.add(new Double[]{dispacementList.get(i),loadList.get(i)});
            }
            Double stroke = Double.parseDouble(nearlyWorkDiagram.getString("STROKE"));
            Double frequency = Double.parseDouble(nearlyWorkDiagram.getString("FREQUENCY"));

        });
    }
}
